﻿Imports System.IO
Imports System.IO.IsolatedStorage

Partial Public Class MainPage
    Inherits PhoneApplicationPage
    ' Tombstoning helpers
    Private radioButtonIndex As Integer = -1
    Private mediaStartState As MediaElementState = MediaElementState.Playing
    Private mediaStartPosition As TimeSpan = TimeSpan.Zero

    ' To prevent Slider reentry
    Private settingSliderFromPlay As Boolean

    ' Constructor
    Public Sub New()
        InitializeComponent()

        ' Set fields of appbar buttons; otherwise they're null
        rewindButton = TryCast(Me.ApplicationBar.Buttons(0), ApplicationBarIconButton)
        playButton = TryCast(Me.ApplicationBar.Buttons(1), ApplicationBarIconButton)
        pauseButton = TryCast(Me.ApplicationBar.Buttons(2), ApplicationBarIconButton)
        endButton = TryCast(Me.ApplicationBar.Buttons(3), ApplicationBarIconButton)

        ' Check if file is in isolated storage; otherwise start downloading it
        Using isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
            If isoStore.FileExists(TryCast(isoStoreRadioButton.Tag, String)) Then
                isoStoreRadioButton.IsEnabled = True
            Else
                Dim webClient As New WebClient
                AddHandler webClient.OpenReadCompleted, AddressOf OnWebClientOpenReadCompleted
                webClient.OpenReadAsync(New Uri("http://www.archive.org/download/BrahmsViolinConcerto-Heifetz/03Iii.AllegroGiocosoMaNonTroppoVivace.mp3"))
            End If
        End Using

        ' Update the elapsed time and slider position
        AddHandler CompositionTarget.Rendering, Sub(sender, args)
                                                    Dim pos As TimeSpan = mediaElement.Position
                                                    Dim dur As TimeSpan = mediaElement.NaturalDuration.TimeSpan

                                                    elapsedTime.Text = pos.ToString("m\:ss\.f")
                                                    totalTime.Text = dur.ToString("m\:ss\.f")
                                                    mediaState.Text = mediaElement.CurrentState.ToString

                                                    settingSliderFromPlay = True

                                                    If dur.Ticks > 0 Then
                                                        slider.Value = CDbl(pos.Ticks) / dur.Ticks
                                                    Else
                                                        slider.Value = 0
                                                    End If

                                                    settingSliderFromPlay = False
                                                End Sub
    End Sub

    ' When the music file is downloaded, save it to isolated storage
    Private Sub OnWebClientOpenReadCompleted(ByVal sender As Object, ByVal args As OpenReadCompletedEventArgs)
        If (Not args.Cancelled) AndAlso args.Error Is Nothing Then
            Dim inpStream As Stream = args.Result
            Dim buffer(inpStream.Length - 1) As Byte
            inpStream.Read(buffer, 0, buffer.Length)

            Using isoStore As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
                Dim isoPathName As String = TryCast(isoStoreRadioButton.Tag, String)
                Dim isoDirName As String = Path.GetDirectoryName(isoPathName)

                If Not isoStore.DirectoryExists(isoDirName) Then
                    isoStore.CreateDirectory(isoDirName)
                End If

                Using isoStream As IsolatedStorageFileStream = isoStore.CreateFile(isoPathName)
                    isoStream.Write(buffer, 0, buffer.Length)
                    isoStoreRadioButton.IsEnabled = True
                End Using
            End Using
        End If
    End Sub

    Private Sub OnRadioButtonChecked(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim radioButton As RadioButton = TryCast(sender, RadioButton)
        Dim uriString As String = TryCast(radioButton.Tag, String)

        ' Save index for tombstoning
        radioButtonIndex = radioButtonPanel.Children.IndexOf(radioButton)

        If radioButton Is isoStoreRadioButton Then
            ' Call SetSource on MediaElement using isolated-storage stream.
            Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
                Using isoStream As Stream = storage.OpenFile(uriString, FileMode.Open)
                    mediaElement.SetSource(isoStream)
                End Using
            End Using
        Else
            ' Set Source property on MediaElement using URI
            Dim uri As New Uri(uriString, If(uriString.Contains(":"c), UriKind.Absolute, UriKind.Relative))
            mediaElement.Source = uri
        End If
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        If radioButtonIndex <> -1 Then
            Me.State.Add("radioButtonIndex", radioButtonIndex)
            Me.State.Add("mediaElementCurrentState", mediaElement.CurrentState)
            Me.State.Add("mediaElementPosition", mediaElement.Position)

            ' Uncheck checked RadioButton
            TryCast(radioButtonPanel.Children(radioButtonIndex), RadioButton).IsChecked = False
            radioButtonIndex = -1
        End If
        MyBase.OnNavigatedFrom(args)
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        If Me.State.ContainsKey("radioButtonIndex") Then
            ' Get the entries
            Dim radioButtonIndex As Integer = CInt(Fix(Me.State("radioButtonIndex")))
            mediaStartState = CType(Me.State("mediaElementCurrentState"), MediaElementState)
            mediaStartPosition = CType(Me.State("mediaElementPosition"), TimeSpan)

            ' Remove the entries
            Me.State.Remove("radioButtonIndex")
            Me.State.Remove("mediaElementPosition")
            Me.State.Remove("mediaElementCurrentState")

            ' Start the ball rolling
            TryCast(radioButtonPanel.Children(radioButtonIndex), RadioButton).IsChecked = True
        End If

        MyBase.OnNavigatedTo(args)
    End Sub

    Private Sub OnMediaElementMediaOpened(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Select Case mediaStartState
            Case MediaElementState.Playing
                mediaElement.Play()
            Case MediaElementState.Paused
                mediaElement.Pause()
        End Select
        mediaElement.Position = mediaStartPosition

        ' Set fields to default values
        mediaStartState = MediaElementState.Playing
        mediaStartPosition = TimeSpan.Zero
    End Sub

    Private Sub OnMediaElementMediaFailed(ByVal sender As Object, ByVal args As ExceptionRoutedEventArgs)
        errorTextBlock.Text = args.ErrorException.Message

        If radioButtonIndex <> -1 Then
            ' Uncheck checked RadioButton
            TryCast(radioButtonPanel.Children(radioButtonIndex), RadioButton).IsChecked = False
            radioButtonIndex = -1
        End If
    End Sub

    Private Sub OnMediaElementCurrentStateChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Select Case mediaElement.CurrentState
            Case MediaElementState.Playing
                rewindButton.IsEnabled = True
                playButton.IsEnabled = False
                pauseButton.IsEnabled = True
                endButton.IsEnabled = True

            Case MediaElementState.Paused, MediaElementState.Stopped
                rewindButton.IsEnabled = True
                playButton.IsEnabled = True
                pauseButton.IsEnabled = False
                endButton.IsEnabled = True

            Case Else
                rewindButton.IsEnabled = False
                playButton.IsEnabled = False
                pauseButton.IsEnabled = False
                endButton.IsEnabled = False
        End Select
    End Sub

    Private Sub OnSliderValueChanged(ByVal sender As Object, ByVal args As RoutedPropertyChangedEventArgs(Of Double))
        If Not settingSliderFromPlay Then
            Dim ticks As Long = CLng(Fix(args.NewValue * mediaElement.NaturalDuration.TimeSpan.Ticks))
            mediaElement.Position = New TimeSpan(ticks)
        End If
    End Sub

    Private Sub OnRewindButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        mediaElement.Position = TimeSpan.Zero
    End Sub

    Private Sub OnPlayButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        mediaElement.Play()
    End Sub

    Private Sub OnPauseButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        mediaElement.Pause()
    End Sub

    Private Sub OnEndButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        mediaElement.Position = mediaElement.NaturalDuration.TimeSpan
    End Sub
End Class
